def source_paths
  Array(super) + 
    [File.expand_path(File.dirname(__FILE__))]
end

# Add needed gem to Gemfile
gsub_file 'Gemfile', 'https://rubygems.org', 'https://gems.ruby-china.com'
inject_into_file 'Gemfile', after: "source 'https://gems.ruby-china.com'" do <<-GEM_COMMENT

# A Ruby gem to load environment variables from `.env`.
# https://github.com/bkeepers/dotenv
# If you use gems that require environment variables to be set before they are loaded, then list dotenv-rails in the Gemfile before those other gems and require dotenv/rails-now.
gem 'dotenv-rails', require: 'dotenv/rails-now'
GEM_COMMENT
end

# gem 'sqlite3', '~> 1.3.6'

gem 'pg'
inject_into_file 'Gemfile', before: "gem 'pg" do <<-COMMENT
# Use PostgresSql as the database for deploy
COMMENT
end

# gem 'jbuilder'
# inject_into_file 'Gemfile', before: "gem 'jbuilder'" do <<-COMMENT
# # Build JSON APIs with ease. Read more: https://github.com/rails/jbuilder
# COMMENT
# end
#
# gem 'rack-cors'
# inject_into_file 'Gemfile', before: "gem 'rack-cors" do <<-COMMENT
# # Use Rack CORS for handling Cross-Origin Resource Sharing (CORS), making cross-origin AJAX possible
# COMMENT
# end
#
# gem 'redis'

gem 'uuid64'
inject_into_file 'Gemfile', before: "gem 'uuid64'" do <<-COMMENT
# This is a library generating unique id in short pattern. https://rubygems.org/gems/uuid64
# https://github.com/heckpsi-lab/uuid64
COMMENT
end

gem 'responders'
inject_into_file 'Gemfile', before: "gem 'responders'" do <<-COMMENT
# A set of Rails responders to dry up your application http://blog.plataformatec.com.br/
# https://github.com/plataformatec/responders
# rails g responders:install
COMMENT
end
gsub_file 'app/controllers/application_controller.rb', 'respond_to :html', 'respond_to :json'

gem 'will_paginate'
inject_into_file 'Gemfile', before: "gem 'will_paginate'" do <<-COMMENT
# Pagination library for Rails, Sinatra, Merb, DataMapper, and more http://github.com/mislav/will_paginat…
# https://github.com/mislav/will_paginate
COMMENT
end

gem 'aasm'
inject_into_file 'Gemfile', before: "gem 'aasm" do <<-COMMENT
# AASM - State machines for Ruby classes (plain Ruby, ActiveRecord, Mongoid)
# https://github.com/aasm/aasm
# rails generate aasm NAME [COLUMN_NAME]
COMMENT
end

# A Ruby static code analyzer, based on the community Ruby style guide. http://rubocop.readthedocs.io
# https://github.com/bbatsov/rubocop
gem 'rubocop', require: false
gem 'rubocop-rails'
gem 'rubocop-performance'

gem 'simple_controller', git: 'https://git.tallty.com/open-source/simple_controller.git'
gem 'acts_as_pasting', git: 'http://git.tallty.com/open-source/acts_as_pasting.git'
gem 'tallty_import_export', git: 'https://git.tallty.com/open-source/tallty_import_export.git'
gem 'acts_as_paranoid'
gem "audited"
gem 'pry-byebug'

# deploy
gem 'mina', require: false
gem 'mina-multistage', require: false

gem 'sprockets'

gem_group :development, :test do
  gem 'rspec-rails'
  gem 'shoulda-matchers'
  gem 'factory_bot_rails'
  gem 'rspec-rails-swagger', git: "https://git.tallty.com/open-source/rspec-rails-swagger.git"
  gem 'simplecov', require: false
  gem 'annotate'
  gem "pry-rails"
end

# .ignore files
inject_into_file '.gitignore', after: ".byebug_history" do <<-COMMENT
# Ignore swagger files
/swagger/*
# Ignore simplecov coverage files
/coverage/*
# Ignore puma running files
/.pids/*
!/pids/.keep
# Other ignore files
dump.rdb
*.pem
# Ignore public folder
/public/*
# Ignore local files
*.local
.DS_Store
COMMENT
end

after_bundle do
  run "spring stop"
  # rspec-rails gem install
  generate 'rspec:install'
  # rspec-rails-swagger gem install
  generate 'rspec:swagger_install'
  inject_into_file 'Gemfile', before: "gem 'rspec-rails-swagger" do <<-COMMENT
  # Generate Swagger 2.0 docs for Rails apps using RSpec request specs. Test results can be captured as response examples.
  # https://github.com/drewish/rspec-rails-swagger
  # rails generate rspec:swagger_install
  # rails generate rspec:swagger PostsController
  # bundle exec rake swagger
  COMMENT
  end
  # Audited (formerly acts_as_audited) is an ORM extension that logs all changes to your Rails models.
  generate 'audited:install'
  # shoulda-matchers gem configuration
  copy_file 'src/spec/support/shoulda_matches.rb', 'spec/support/shoulda_matches.rb'
  # factory_bot_rails gem configuration
  gsub_file 'spec/rails_helper.rb',
            "# Dir[Rails.root.join('spec', 'support', '**', '*.rb')].each { |f| require f }",
            "Dir[Rails.root.join('spec', 'support', '**', '*.rb')].each { |f| require f }"
  copy_file 'src/spec/support/factory_bot.rb', 'spec/support/factory_bot.rb'

  # simplecov configuration
  inject_into_file 'spec/rails_helper.rb', before: /^RSpec.configure/ do <<-COMMENT
  require 'simplecov'
  SimpleCov.start

  COMMENT
  end
  # rubocop configuration
  copy_file 'src/.rubocop.yml', '.rubocop.yml'
  copy_file 'src/.env', '.env'
  copy_file 'src/.env.test', '.env.test'
  copy_file 'src/.env.test.local', '.env.test.local'
  copy_file 'src/.rubocop.yml', '.rubocop.yml'
  remove_file 'Rakefile'
  copy_file 'src/Rakefile', 'Rakefile'
  # ci configuration
  copy_file 'src/.gitlab-ci.yml', '.gitlab-ci.yml'
  gsub_file '.gitlab-ci.yml', 'mysql_test_database', "#{app_name}_test_db"
  # puma configuration
  remove_file 'config/puma.rb'
  copy_file 'src/config/puma.rb', 'config/puma.rb'
  remove_file 'config/database.yml'
  copy_file 'src/config/database.yml', 'config/database.yml'
  copy_file 'src/config/ohm.rb', 'config/initializers/ohm.rb'
  remove_file 'config/initializers/cors.rb'
  copy_file 'src/config/cors.rb', 'config/initializers/cors.rb'
  remove_file 'spec/swagger_helper.rb'
  copy_file 'src/spec/swagger_helper.rb', 'spec/swagger_helper.rb'
  remove_file 'app/controllers/application_controller.rb'
  copy_file 'src/app/controllers/application_controller.rb', 'app/controllers/application_controller.rb'
  copy_file 'src/config/audited.rb', 'config/initializers/audited.rb'

  gsub_file 'config/puma.rb', 'app_name_to_replace', app_name
  gsub_file 'spec/swagger_helper.rb', 'app_name_to_replace', app_name
  gsub_file 'Rakefile', 'app_name_to_replace', app_name
  gsub_file '.env', 'app_name_to_replace', app_name
  gsub_file '.env.test', 'app_name_to_replace', app_name
  gsub_file '.env.test.local', 'app_name_to_replace', app_name

  # deploy
  copy_file 'src/config/deploy.rb', 'config/deploy.rb'
  directory 'src/config/deploy', 'config/deploy'

  # run 'mkdir .pids'
  # run 'touch .pids/.keep'
  run 'bundle binstubs rspec-core'
  run 'bundle binstubs bundler --force'

  git :init
  git add: '.'
  git commit: "-a -m 'Initial Rails API Project'"
end
